/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "global.h"
#include "docklet.h"


int
main (int argc, char *argv[])
{
	pid_t child; // to go in bacground
	GtkWidget *interf,*InterfaceNameInPref;
	int count;
	gchar *ForceInterfaceName = NULL; // for --device option
	
#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif
	
  // mynetmonitor --version
  if(argc>1)
  {
	  if(!strcmp("--version",argv[1]))
	  {
		  printf("MyNetMonitor - Diego Bazzanella 2003 - Version 1.0.1\n");
		  exit(0);
	  }
  }
  
  // mynetmonitor --help
  if(argc>1)
  {
	  if(!strcmp("--help",argv[1]))
	  {
		  printf("mynetmonitor --lang xx			force language to xx\n");
		  printf("mynetmonitor --device xxxx		force device to xxxx\n");
		  printf("mynetmonitor --version			show version and quit\n");
		  printf("mynetmonitor --help   			show this message and quit\n");
		  exit(0);
	  }
  }
  
  // mynetmonitor --lang xx
  for(count = 0; count < argc; count++)
	  if(!strcmp("--lang",argv[count]))
		  {
			  printf("Forcing language to %s\n",argv[count + 1]);
			  printf("(if %s is not supported force to english en)\n",argv[count + 1]);
			  setenv("LANGUAGE",argv[count + 1],1);
		  }
  
  // mynetmonitor --device xxx
  for(count = 0; count < argc; count++)
	  if(!strcmp("--device",argv[count]))
		  {
			  if (argc > count + 1)
			  {
				  ForceInterfaceName = argv [count + 1];
				  printf("Forcing device to %s\n",argv[count + 1]);
			  }
		  }

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");  
  
  // create all windows
  mainWindow  = create_mainWindow ();
  infoWindow  = create_infoWindow ();
  errorDialog = create_errorDialog ();
  errorMessage = lookup_widget(errorDialog,"errorMessage");
  tooltips = gtk_tooltips_new ();
  
  // adjust justify in some labels (not in interfece.c to overwrite glade code)
  GtkWidget *lab1,*lab2,*lab3;
  lab1 = lookup_widget(mainWindow,"label1");
  lab2 = lookup_widget(mainWindow,"label2");
  lab3 = lookup_widget(mainWindow,"label3");
  gtk_label_set_justify(GTK_LABEL(lab1),GTK_JUSTIFY_CENTER);
  gtk_label_set_justify(GTK_LABEL(lab2),GTK_JUSTIFY_CENTER);
  gtk_label_set_justify(GTK_LABEL(lab3),GTK_JUSTIFY_CENTER);
	
  // Go in background
    child = fork();
    if (child  > 0)
    {
	    exit(0);	// Parent exits
    }
  
  // Load pref.
  set_pref_var();
	
  // for --device option
  if (ForceInterfaceName != NULL)
  {
	  InterfaceNameInPref = lookup_widget(mainWindow,"interfaceName");
	  gtk_entry_set_text(GTK_ENTRY(InterfaceNameInPref),ForceInterfaceName);
  }
		
  // put icon in system tray
  docklet_register_icon_factory();
  docklet_create(mainWindow);
  
  // for wait status icon
  newonline=0;

  // check if connection falls
  okdown = 0;	
	
  // init: disconnect command is not running
  disconnectCommandRunning = 0;
	
  // init of interfacenameGlobal
  interf = lookup_widget(mainWindow,"interfaceName");
  interfacenameGlobal = g_build_filename((gchar *)(gtk_entry_get_text(GTK_ENTRY(interf))),NULL);

  // call function refresh each second
  gtk_timeout_add(1000,(GtkFunction) refresh,NULL);

  gtk_main ();
  return 0;
}


// Load pref.
void set_pref_var(void)
{
	GtkWidget *interfaceName,*connectCommand,*disconnectCommand;
	FILE *file_descriptor;
	gchar *filename,*prefInterfaceName,*prefConnectCommand,*prefDisconnectCommand;
	
	#define MAX_STRING_LENGTH 100
	
	// Load default
	interfaceName = lookup_widget(mainWindow,"interfaceName");
	gtk_entry_set_text(GTK_ENTRY(interfaceName),"ppp0");
	interfaceNameOld = g_build_filename("ppp0", NULL);
	
	disconnectCommand = lookup_widget(mainWindow,"disconnectCommand");
	gtk_entry_set_text(GTK_ENTRY(disconnectCommand),"/usr/bin/ppp-off");
	disconnectCommandOld = g_build_filename("/usr/bin/ppp-off", NULL);
	
	connectCommand = lookup_widget(mainWindow,"connectCommand");
	gtk_entry_set_text(GTK_ENTRY(connectCommand),"/usr/bin/ppp-on");
	connectCommandOld = g_build_filename("/usr/bin/ppp-on", NULL);
	
	// Load from pref. file
	filename = g_build_filename( (gchar *)getenv("HOME"), ".mynetmonitor" , NULL);
	if((file_descriptor = fopen(filename,"r"))==NULL)
	{
		// file not exists, let's create default pref. file
	    if((file_descriptor = fopen(filename,"w"))==NULL)
	    {
		//printf("Error: I can't create pref. file\n");
		gtk_label_set_text(GTK_LABEL(errorMessage),_("Error: I can't create pref. file"));
		gtk_widget_show(errorDialog);
	    }
		else
		{
			// writing default values in created pref. file
			fprintf(file_descriptor,
			        "%s\n%s\n%s\n",
					g_build_filename("ppp0", NULL),
					g_build_filename("/usr/bin/ppp-on", NULL),
					g_build_filename("/usr/bin/ppp-off", NULL));
			
			fclose(file_descriptor);
		}
	}
	// else file already exists, then read the values
	else
	{
		prefInterfaceName = g_malloc (MAX_STRING_LENGTH + 2);
		prefConnectCommand = g_malloc (MAX_STRING_LENGTH + 2);
		prefDisconnectCommand = g_malloc (MAX_STRING_LENGTH + 2);
		
		if(((prefInterfaceName = fgets (prefInterfaceName, MAX_STRING_LENGTH + 2, file_descriptor))
				!= NULL))
		{
			prefInterfaceName[strlen (prefInterfaceName) - 1] = 0;
			gtk_entry_set_text(GTK_ENTRY(interfaceName),(const gchar *)prefInterfaceName);
			interfaceNameOld = g_build_filename(prefInterfaceName, NULL);
		}
		
		if(((prefConnectCommand = fgets (prefConnectCommand, MAX_STRING_LENGTH + 2, file_descriptor))
				!= NULL))
		{
			prefConnectCommand[strlen (prefConnectCommand) - 1] = 0;
			gtk_entry_set_text(GTK_ENTRY(connectCommand),(const gchar *)prefConnectCommand);
			connectCommandOld = g_build_filename(prefConnectCommand, NULL);
		}
		
		if(((prefDisconnectCommand = fgets (prefDisconnectCommand, MAX_STRING_LENGTH + 2, file_descriptor))
				!= NULL))
		{
			prefDisconnectCommand[strlen (prefDisconnectCommand) - 1] = 0;
			gtk_entry_set_text(GTK_ENTRY(disconnectCommand),(const gchar *)prefDisconnectCommand);
			disconnectCommandOld = g_build_filename(prefDisconnectCommand, NULL);
		}
		
	}
}

void save_pref(void)
{
	GtkWidget *interfaceName,*connectCommand,*disconnectCommand;
	FILE *file_descriptor;
	gchar *filename,*prefInterfaceName,*prefConnectCommand,*prefDisconnectCommand;
	
	filename = g_build_filename( (gchar *)getenv("HOME"), ".mynetmonitor" , NULL);
	if((file_descriptor = fopen(filename,"w"))==NULL)
	{
		//printf("Error: I can't save pref. file\n");
		gtk_label_set_text(GTK_LABEL(errorMessage),_("Error: I can't save pref. file"));
		gtk_widget_show(errorDialog);
		return;
	}
	else
	{
		interfaceName = lookup_widget(mainWindow,"interfaceName");
		prefInterfaceName = (gchar *)(gtk_entry_get_text(GTK_ENTRY(interfaceName)));
		
		connectCommand = lookup_widget(mainWindow,"connectCommand");
		prefConnectCommand = (gchar *)(gtk_entry_get_text(GTK_ENTRY(connectCommand)));
		
		disconnectCommand = lookup_widget(mainWindow,"disconnectCommand");
		prefDisconnectCommand = (gchar *)(gtk_entry_get_text(GTK_ENTRY(disconnectCommand)));
		
		fprintf(file_descriptor,
			        "%s\n%s\n%s\n",
					g_build_filename(prefInterfaceName,NULL),
					g_build_filename(prefConnectCommand,NULL),
					g_build_filename(prefDisconnectCommand,NULL));
			
		fclose(file_descriptor);
	}
}

// call function 'refresh' each second
void refresh(void)
{
	gint isonlineold,sent,received,sent_tooltips,received_tooltips,hours,minutes,sec;
	gchar *stringa;
	
	isonlineold = isonline;
	isonline = read_conn(&sent,&received);
	
	// if disconnect command is running check timeout
	if ( disconnectCommandRunning > 0) checkDisconnectCommandRunning ();
	
    if(isonline == 1)
    {	
		 // for wait status icon
		if(newonline==0 && received != 0)
		{
			newonline = 1;
			//printf(_("Online"));
			//printf("\n");
			docklet_update_status((enum docklet_status)online);
			gtk_tooltips_set_tip (	tooltips,
									box,
									_("Online"),
									NULL);
		}
		 // online and status changed in the last second
		 if (isonline != isonlineold)
		 {
			 if(status != wait)
			 {
				 docklet_update_status((enum docklet_status)wait);
				 gtk_tooltips_set_tip (	tooltips,
									box,
									_("Waiting"),
									NULL);
			 }
			 sentold  = 0;
			 sentold1 = 0;
		     sentold2 = 0;
			 sentold3 = 0;
			 receivedold  = 0;
		     receivedold1 = 0;
		     receivedold2 = 0;
			 receivedold3 = 0;
			 seconds = 0;
		 }
		 // online and not changed in the last second
		 else
		 {  
			 // calculate byte sent and receive and set icon
			 sent_tooltips=(sent-sentold);
			 received_tooltips=(received-receivedold);
			
			 sentold3 = sent_tooltips;
			 receivedold3 = received_tooltips;
			
			 sent_tooltips = (sent_tooltips + sentold1 + (sentold2 * 2)) / 4;
			 received_tooltips = (received_tooltips + receivedold1 + (receivedold2 * 2)) / 4;
			 
			 if(((((float)(received-receivedold))/1024) <= 0.2) && ((((float)(sent-sentold))/1024) > 0.2))
			 {
				 if(status != send)
				 {
					 docklet_update_status((enum docklet_status)send); 
				 }
			 }
			 
			 if(((((float)(received-receivedold))/1024) > 0.2) && ((((float)(sent-sentold))/1024) <= 0.2))
			 {
				 if(status != receive)
				 {
					 docklet_update_status((enum docklet_status)receive); 
				 }
			 }
			 
			 if(((((float)(received-receivedold))/1024) > 0.2) && ((((float)(sent-sentold))/1024) > 0.2))
			 {
				 if(status != sendreceive)
				 {
					 docklet_update_status((enum docklet_status)sendreceive); 
				 }
			 }
			 
			 if(((((float)(received-receivedold))/1024) <= 0.2) && ((((float)(sent-sentold))/1024) <= 0.2))
			 {
				 if(status != wait)
				 {
					 docklet_update_status((enum docklet_status)online); 
				 }
			 }
			
			 sentold = sent;
			 receivedold = received;
			
			 sentold1 = sentold2;
			 sentold2 =  sentold3;
			 receivedold1 = receivedold2;
			 receivedold2 = receivedold3;
			
			 // set time passed by
			 seconds=seconds+1;		
			 hours = seconds / (60*60);
			 minutes = (seconds - (hours*60*60)) / 60;
			 sec = seconds - (hours*60*60) - (minutes*60);
			
			 stringa = g_strdup_printf(_("Online              \nSent:%.1fK \nReceived: %.1fK \nTime: %.2d:%.2d:%.2d"),
										((float)(sent_tooltips)/1024),((float)(received_tooltips)/1024),hours,minutes,sec);
			 stringa = g_strdup_printf("%s %s",interfacenameGlobal,stringa);
			
			 // set tooltip
			 gtk_tooltips_set_tip (	tooltips,
									box,
									(const gchar *)stringa,
									NULL);
									
		}
								
		
	}
	// else status is offline
    else
    {	
		// for wait status icon
		newonline = 0;
		
		// if runneded disconnect command ok
		if ( disconnectCommandRunning > 0 ) disconnectCommandRunning = 0;
		
		// check if connection falls
		if ( (isonline != isonlineold) && (okdown != 1) )
		{
			gtk_label_set_text(GTK_LABEL(errorMessage),_("Attention: connection is down"));
        	gtk_widget_show(errorDialog);
		}
		if( okdown != 0 ) okdown = 0;
		
		if(isonline == 2)
		{
			//printf("Error: no /proc/net/dev file\n");
			gtk_label_set_text(GTK_LABEL(errorMessage),_("Error: no /proc/net/dev file"));
        	gtk_widget_show(errorDialog);
		}
		else
		{
			stringa = g_strdup_printf("%s %s",interfacenameGlobal,_("Offline"));
			gtk_tooltips_set_tip (tooltips, box,stringa,NULL);
			if (isonline != isonlineold)
				if(status != offline)
				{
					docklet_update_status((enum docklet_status)offline);
				}
		}
    }
}


/* READ_CONN returns 0 if PPP is down, or returns 1 and the current amount
   of bytes sent and received are returned in the pointers if PPP is up*/
gint read_conn(gint *sent,gint *received)
{
  FILE *procnetdev=NULL;
  G_CONST_RETURN gchar *interfaceName;

  guchar caracter,cadena[1024],cadena2[8],found;
  gint basura;
	
	
  //interfaceName = gtk_entry_get_text(GTK_ENTRY(lookup_widget(mainWindow,"interfaceName")));

  interfaceName = g_build_filename(interfacenameGlobal,NULL);
	
  // rewind(procnetdev); // return to the start of file
  procnetdev=fopen("/proc/net/dev","rb");
  if(procnetdev==NULL)
    return 2;

  switch(strlen(interfaceName)) {
  case 1:
    sprintf(cadena2,"     %s:",interfaceName);
    break;
  case 2:
    sprintf(cadena2,"    %s:",interfaceName);
    break;
  case 3:
    sprintf(cadena2,"   %s:",interfaceName);
    break;
  case 4:
    sprintf(cadena2,"  %s:",interfaceName);
    break;
  default:
    return 0;
  }

  found=0;
  do {
    do
      fread(&caracter,1,1,procnetdev);
    while((caracter!=10)&&(!feof(procnetdev))); // jump to next carriage return

    if(feof(procnetdev))
      break; // if we reach EndOfFile, there's no connection

    fread(cadena,7,1,procnetdev); // read the iface identification
    cadena[7]=0;
    if(0==strcmp(cadena,cadena2)) { // found?
      found=1; // yes
      fscanf(procnetdev,"%i",received); // read the number of received bytes
      fscanf(procnetdev,"%i %i %i %i %i %i %i",&basura,&basura,&basura,&basura,&basura,&basura,&basura); // jump over the other numbers
      fscanf(procnetdev,"%i",sent); // and read the number of sent bytes
    }
  } while((found==0)&&(!feof(procnetdev)));

  // printf("retorno: %d %d\n",*sent,*received);

  fclose(procnetdev);

  return (found);
}


// check if dev is offline after the disconnect command, give error after 5 seconds 
void checkDisconnectCommandRunning (void)
{
	if(isonline == 1) disconnectCommandRunning++;
	if(isonline == 1 && disconnectCommandRunning > 6)
	{
		okdown = 0;
		disconnectCommandRunning = 0;
		gtk_label_set_text(GTK_LABEL(errorMessage),_("Problem: disconnect command seems to not work good"));
        gtk_widget_show(errorDialog);
	}
}
